/******************************************************************************
 *    ActReactService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IActReactSrv>::Name = "ActReactService";

	const char * 
	ScriptService<IActReactSrv>::DocStrings[] = {
		"ActReactService()\n\t"
			"Interface to the Act/React system of stimulus and response.",
		"React(kind,intensity,source,dest,...) -> Integer\n\t"
			"Run a reaction response.  kind is an integer, usually from\n\t"
			"GetReactionNamed(). Can be given up to eight optional arguments,\n\t"
			"the type of which depend on the reaction kind.\n\t"
			"Returns 0 on success, or E_FAIL if IReactions isn't available.",
#if (_DARKGAME == 1)
		"Stimulate(sensor,source,intensity) -> Integer\n\t"
			"Stimulate sensor with source at intensity.\n\t"
			"intensity is a float. All the others are objects.\n\t"
			"Returns 0 on success, otherwise 1.",
#else
		"Stimulate(sensor,source,intensity,obj) -> Integer\n\t"
			"Stimulate sensor with source at intensity.\n\t"
			"intensity is a float. All the others are objects.\n\t"
			"obj is only used in Thief 2.\n\t"
			"Returns 0 on success, otherwise 1.",
#endif
		"GetReactionNamed(name) -> Integer\n\t"
			"Returns the ID of the reaction with the given name.",
		"GetReactionName(kind) -> String\n\t"
			"Returns the name of a reaction type.",
		"SubscribeToStimulus(obj,stim) -> Integer\n\t"
			"Register obj to receive the stimulus messages for stim.\n\t"
			"obj must be concrete, and stim must be an archetype.\n\t"
			"Returns 0 on success, E_FAIL otherwise.",
		"UnsubscribeToStimulus(obj,stim) -> Integer\n\t"
			"Stop listening for stimulus messages for stim.\n\t"
			"obj must be concrete, and stim must be an archetype.\n\t"
			"Returns 0 on success, E_FAIL otherwise.",
		"BeginContact(source,sensor) -> Integer\n\t"
			"Initiates contact stimulus on sensor using source.\n\t"
			"Returns 0 on success.",
		"EndContact(source,sensor) -> Integer\n\t"
			"Terminates contact stimulus on sensor using source.\n\t"
			"Returns 0 on success.",
		"SetSingleSensorContact(source,sensor) -> Integer\n\t"
			"Touch sensor using source.\n\t"
			"Returns 0 on success.",
	};

	const PyMethodDef 
	ScriptService<IActReactSrv>::Methods[] = {
		{	"React", 
			reinterpret_cast<PyCFunction>(React), 
			METH_VARARGS, 
			const_cast<char*>(DocStrings[1])
		},
		{	"Stimulate", 
			reinterpret_cast<PyCFunction>(Stimulate), 
			METH_VARARGS, 
			const_cast<char*>(DocStrings[2])
		},
		{	"GetReactionNamed",
			reinterpret_cast<PyCFunction>(GetReactionNamed), 
			METH_VARARGS, 
			const_cast<char*>(DocStrings[3])
		},
		{	"GetReactionName", 
			reinterpret_cast<PyCFunction>(GetReactionName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"SubscribeToStimulus",
			reinterpret_cast<PyCFunction>(SubscribeToStimulus),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"UnsubscribeToStimulus", 
			reinterpret_cast<PyCFunction>(UnsubscribeToStimulus),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"BeginContact",
			reinterpret_cast<PyCFunction>(BeginContact),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"EndContact",
			reinterpret_cast<PyCFunction>(EndContact),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"SetSingleSensorContact",
			reinterpret_cast<PyCFunction>(SetSingleSensorContact),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IActReactSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IActReactSrv>::Name),
		sizeof(ScriptServiceObject<IActReactSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IActReactSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IActReactSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IActReactSrv> * 
	ScriptService<IActReactSrv>::NewObject(PyObject *)
	{
		IActReactSrv *iface = static_cast<IActReactSrv*>(g_pScriptManager->GetService(IID_IActReactScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IActReactSrv> *obj = PyObject_New(ScriptServiceObject<IActReactSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IActReactSrv>::React(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		long kind;
		object src, dest;
		float intensity;
		cMultiParm data[8];

		if (!PyArg_ParseTuple(args, "lfO&O&:React", &kind, &intensity, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		if (PyTuple_GET_SIZE(args) > 12)
		{
			PyErr_SetString(PyExc_TypeError, "too many arguments");
			return NULL;
		}
		int argc = PyTuple_GET_SIZE(args) - 4;
		for (int n = 0; n < argc; ++n)
		{
			PyObject *obData = PyTuple_GET_ITEM(args, n + 4);
			if (!MultiParmFromObject(data[n], obData))
			{
				while (n--)
					data[n].Free();
				PyErr_SetString(PyExc_TypeError, "argument must be an Integer, Float, String, or Vector");
				return NULL;
			}
		}

		long ret = self->m_pIFace->React(kind, intensity, src, dest,
						data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
		data[0].Free(); data[1].Free(); data[2].Free(); data[3].Free();
		data[4].Free(); data[5].Free(); data[6].Free(); data[7].Free();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::Stimulate(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		float intensity;
		object source, sensor;
#if (_DARKGAME == 1)
		if (!PyArg_ParseTuple(args, "O&O&f:Stimulate", cvtObject, &source, cvtObject, &sensor, &intensity))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Stimulate(source, sensor, intensity);
#else
		int type;
		if (!PyArg_ParseTuple(args, "O&O&fO&:Stimulate", cvtObject, &source, cvtObject, &sensor, &intensity, cvtObject, &type))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Stimulate(source, sensor, intensity, type);
#endif
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::GetReactionNamed(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:GetReactionNamed", &name))
		{
			return NULL;
		}
		int type = self->m_pIFace->GetReactionNamed(name);
		return PyInt_FromLong(type);
	}

	PyObject * 
	ScriptService<IActReactSrv>::GetReactionName(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		long kind;
		if (!PyArg_ParseTuple(args, "l:GetReactionName", &kind))
		{
			return NULL;
		}
		cScrStr ret;
		self->m_pIFace->GetReactionName(ret,kind);
		return PyString_FromString(static_cast<const char*>(ret));
	}

	PyObject * 
	ScriptService<IActReactSrv>::SubscribeToStimulus(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		object obj, stim;
		if (!PyArg_ParseTuple(args, "O&O&:SubscribeToStimulus", cvtObject, &obj, cvtObject, &stim))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SubscribeToStimulus(obj,stim);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::UnsubscribeToStimulus(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		object obj, stim;
		if (!PyArg_ParseTuple(args, "O&O&:UnsubscribeToStimulus", cvtObject, &obj, cvtObject, &stim))
		{
			return NULL;
		}
		long ret = self->m_pIFace->UnsubscribeToStimulus(obj,stim);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::BeginContact(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		object source, sensor;
		if (!PyArg_ParseTuple(args, "O&O&:BeginContact", cvtObject, &source, cvtObject, &sensor))
		{
			return NULL;
		}
		long ret = self->m_pIFace->BeginContact(source,sensor);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::EndContact(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		object source, sensor;
		if (!PyArg_ParseTuple(args, "O&O&:EndContact", cvtObject, &source, cvtObject, &sensor))
		{
			return NULL;
		}
		long ret = self->m_pIFace->EndContact(source,sensor);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IActReactSrv>::SetSingleSensorContact(ScriptServiceObject<IActReactSrv> *self, PyObject *args)
	{
		object source, sensor;
		if (!PyArg_ParseTuple(args, "O&O&:SetSingleSensorContact", cvtObject, &source, cvtObject, &sensor))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SetSingleSensorContact(source,sensor);
		return PyInt_FromLong(ret);
	}

} // namespace PyNumbra

