/******************************************************************************
 *    ContainService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IContainSrv>::Name = "ContainerService";

	const char * 
	ScriptService<IContainSrv>::DocStrings[] = {
		"ContainerService()\n\t"
			"Allows you to modify the contents of a container.",
		"Add(obj,container,type=0,combine=1) -> Integer\n\t"
			"Add obj to the contents of container, removing it from any\n\t"
			"previous container. type is the contain type. If combine is 1,\n\t"
			"the item will attempt to combine with similar objects.",
		"Remove(obj,container) -> Integer\n\t"
			"Remove obj from the contents of container.",
		"MoveAllContents(from,to,combine=1) -> Integer\n\t"
			"Move the contents of from to be contained by to.\n\t"
			"If combine is 1, similar objects will be combined.\n\t"
			"The objects will be given generic contain type.",
#if (_DARKGAME == 2)
		"StackAdd(obj,count) -> Integer\n\t"
			"Adjust the stack count of obj by count, an integer.\n\t"
			"Not available in Thief 1/G.",
		"IsHeld(container,obj) -> Integer\n\t"
			"Checks whether an object is contained by another. If the object\n\t"
			"is in the container, the contain type is returned. If it is\n\t"
			"not contained, then None is returned.\n\t"
			"Not available in Thief 1/G.",
#endif
	};

	const PyMethodDef 
	ScriptService<IContainSrv>::Methods[] = {
		{	"Add",
			reinterpret_cast<PyCFunction>(Add),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"Remove",
			reinterpret_cast<PyCFunction>(Remove),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"MoveAllContents",
			reinterpret_cast<PyCFunction>(MoveAllContents),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
#if (_DARKGAME == 2)
		{	"StackAdd",
			reinterpret_cast<PyCFunction>(StackAdd),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"IsHeld",
			reinterpret_cast<PyCFunction>(IsHeld),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
#endif
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IContainSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IContainSrv>::Name),
		sizeof(ScriptServiceObject<IContainSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IContainSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IContainSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IContainSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kContainTypeGeneric", 0);
		PyModule_AddIntConstant(module, "kContainTypeBelt", -1);
		PyModule_AddIntConstant(module, "kContainTypeHand", -2);
		PyModule_AddIntConstant(module, "kContainTypeAlternate", -3);
	}

	ScriptServiceObject<IContainSrv> * 
	ScriptService<IContainSrv>::NewObject(PyObject *)
	{
		IContainSrv *iface = static_cast<IContainSrv*>(g_pScriptManager->GetService(IID_IContainerScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IContainSrv> *obj = PyObject_New(ScriptServiceObject<IContainSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IContainSrv>::Add(ScriptServiceObject<IContainSrv> *self, PyObject *args)
	{
		object obj, container;
		int type = 0, combine = 1;
		if (!PyArg_ParseTuple(args, "O&O&|ii:Add", cvtObject, &obj, cvtObject, &container, &type, &combine))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Add(obj,container,type,combine);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IContainSrv>::Remove(ScriptServiceObject<IContainSrv> *self, PyObject *args)
	{
		object obj, container;
		if (!PyArg_ParseTuple(args, "O&O&:Remove", cvtObject, &obj, cvtObject, &container))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Remove(obj,container);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IContainSrv>::MoveAllContents(ScriptServiceObject<IContainSrv> *self, PyObject *args)
	{
		object from, to;
		int combine = 1;
		if (!PyArg_ParseTuple(args, "O&O&|i:MoveAllContents", cvtObject, &from, cvtObject, &to, &combine))
		{
			return NULL;
		}
		long ret = self->m_pIFace->MoveAllContents(from,to,combine);
		return PyInt_FromLong(ret);
	}

#if (_DARKGAME == 2)
	PyObject * 
	ScriptService<IContainSrv>::StackAdd(ScriptServiceObject<IContainSrv> *self, PyObject *args)
	{
		object obj;
		int count;
		if (!PyArg_ParseTuple(args, "O&i:StackAdd", cvtObject, &obj, &count))
		{
			return NULL;
		}
		long ret = self->m_pIFace->StackAdd(obj,count);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IContainSrv>::IsHeld(ScriptServiceObject<IContainSrv> *self, PyObject *args)
	{
		object container,obj;
		if (!PyArg_ParseTuple(args, "O&O&:IsHeld", cvtObject, &container, cvtObject, &obj))
		{
			return NULL;
		}
		int ret = self->m_pIFace->IsHeld(container,obj);
		if (ret == MAXLONG)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return PyInt_FromLong(ret);
	}
#endif

} // namespace PyNumbra


