/******************************************************************************
 *    DarkGameService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IDarkGameSrv>::Name = "DarkGameService";

	const char * 
	ScriptService<IDarkGameSrv>::DocStrings[] = {
		"DarkGameService()\n\t"
			"Random non-specific functions.",
		"KillPlayer() -> Integer\n\t"
			"\"Kills\" the player by immobilizing him and tilting the camera.\n\t"
			"Does not actually affect the player object's hit points, nor\n\t"
			"will the mission end.",
		"EndMission() -> Integer\n\t"
			"Immediately ends the mission.",
		"FadeToBlack(time) -> Integer\n\t"
			"Gradually transitions to an all-black screen for the given time\n\t"
			"period. If the time is negative, the screen will immediately\n\t"
			"revert to full visibility.",
#if (_DARKGAME == 2)
		"FoundObject(obj) -> Integer\n\t"
			"Marks a single secret as being discovered. The object must have\n\t"
			"the 'Hidden' flag of 'Dark Gamesys\\Stats' set.\n\t"
			"Only available in Thief 2.",
		"ConfigIsDefined(name) -> Boolean\n\t"
			"Tests whether a config variable has been explicitly defined.\n\t"
			"Only available in Thief 2.",
		"ConfigGetInt(name) -> Integer\n\t"
			"Returns the value of a config variable that is an integer number.\n\t"
			"Only available in Thief 2.",
		"ConfigGetFloat(name) -> Float\n\t"
			"Returns the value of a config variable that is a floating-point\n\t"
			"number.\n\t"
			"Only available in Thief 2.",
		"BindingGetFloat(name) -> Float\n\t"
			"Returns the value of a variable that is defined in the bindings\n\t"
			"(*.BND) file.\n\t"
			"Only available in Thief 2.",
		"GetAutomapLocationVisited(page,room) -> Boolean\n\t"
			"Check whether an area of the automap has been visited.\n\t"
			"Only available in Thief 2.",
		"SetAutomapLocationVisited(page,room) -> Integer\n\t"
			"Marks an area of the automap as visited.\n\t"
			"Only available in Thief 2.",
#endif
	};

	const PyMethodDef 
	ScriptService<IDarkGameSrv>::Methods[] = {
		{	"KillPlayer",
			reinterpret_cast<PyCFunction>(KillPlayer),
			METH_NOARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"EndMission",
			reinterpret_cast<PyCFunction>(EndMission),
			METH_NOARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"FadeToBlack",
			reinterpret_cast<PyCFunction>(FadeToBlack),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
#if (_DARKGAME == 2)
		{	"FoundObject",
			reinterpret_cast<PyCFunction>(FoundObject),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"ConfigIsDefined",
			reinterpret_cast<PyCFunction>(ConfigIsDefined),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"ConfigGetInt",
			reinterpret_cast<PyCFunction>(ConfigGetInt),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"ConfigGetFloat",
			reinterpret_cast<PyCFunction>(ConfigGetFloat),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"BindingGetFloat",
			reinterpret_cast<PyCFunction>(BindingGetFloat),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"GetAutomapLocationVisited",
			reinterpret_cast<PyCFunction>(GetAutomapLocationVisited),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"SetAutomapLocationVisited",
			reinterpret_cast<PyCFunction>(SetAutomapLocationVisited),
			METH_VARARGS,
			const_cast<char*>(DocStrings[10])
		},
#endif
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDarkGameSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDarkGameSrv>::Name),
		sizeof(ScriptServiceObject<IDarkGameSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDarkGameSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDarkGameSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IDarkGameSrv> * 
	ScriptService<IDarkGameSrv>::NewObject(PyObject *)
	{
		IDarkGameSrv *iface = static_cast<IDarkGameSrv*>(g_pScriptManager->GetService(IID_IDarkGameScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDarkGameSrv> *obj = PyObject_New(ScriptServiceObject<IDarkGameSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::KillPlayer(ScriptServiceObject<IDarkGameSrv> *self)
	{
		long ret = self->m_pIFace->KillPlayer();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::EndMission(ScriptServiceObject<IDarkGameSrv> *self)
	{
		long ret = self->m_pIFace->EndMission();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::FadeToBlack(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		float time;
		if (!PyArg_ParseTuple(args, "f:FadeToBlack", &time))
		{
			return NULL;
		}
		long ret = self->m_pIFace->FadeToBlack(time);
		return PyInt_FromLong(ret);
	}

#if (_DARKGAME == 2)
	PyObject * 
	ScriptService<IDarkGameSrv>::FoundObject(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:FoundObject", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->FoundObject(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::ConfigIsDefined(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:ConfigIsDefined", &name))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->ConfigIsDefined(name)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::ConfigGetInt(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		int ret = 0;
		const char *name;
		if (!PyArg_ParseTuple(args, "s:ConfigGetInt", &name))
		{
			return NULL;
		}
		self->m_pIFace->ConfigGetInt(name, &ret);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::ConfigGetFloat(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		float ret = 0.0;
		const char *name;
		if (!PyArg_ParseTuple(args, "s:ConfigGetFloat", &name))
		{
			return NULL;
		}
		self->m_pIFace->ConfigGetFloat(name, &ret);
		return PyFloat_FromDouble(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::BindingGetFloat(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:BindingGetFloat", &name))
		{
			return NULL;
		}
		float ret = self->m_pIFace->BindingGetFloat(name);
		return PyFloat_FromDouble(ret);
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::GetAutomapLocationVisited(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		int page, room;
		if (!PyArg_ParseTuple(args, "ii:GetAutomapLocationVisited", &page, &room))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->GetAutomapLocationVisited(page,room)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IDarkGameSrv>::SetAutomapLocationVisited(ScriptServiceObject<IDarkGameSrv> *self, PyObject *args)
	{
		int page, room;
		if (!PyArg_ParseTuple(args, "ii:SetAutomapLocationVisited", &page, &room))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SetAutomapLocationVisited(page,room);
		return PyInt_FromLong(ret);
	}
#endif


} // namespace PyNumbra


