/******************************************************************************
 *    DoorService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IDoorSrv>::Name = "DoorService";

	const char * 
	ScriptService<IDoorSrv>::DocStrings[] = {
		"DoorService()\n\t"
			"Allows you to operate objects that are doors.",
		"CloseDoor(obj) -> Integer\n\t"
			"Makes the door go to the closed state.",
		"OpenDoor(obj) -> Integer\n\t"
			"Makes the door go to the open state.",
		"GetDoorState(obj) -> Integer\n\t"
			"Returns the current state of the door. This is not the same as\n\t"
			"the door action, which is used in the door related script\n\t"
			"messages.",
		"ToggleDoor(obj) -> Integer\n\t"
			"Switches the door action between open/opening and close/closing.",
#if (_DARKGAME == 2)
		"SetBlocking(obj,on) -> Integer\n\t"
			"Only available in Thief 2.",
		"GetSoundBlocking(obj) -> Boolean\n\t"
			"Only available in Thief 2.",
#endif
	};

	const PyMethodDef 
	ScriptService<IDoorSrv>::Methods[] = {
		{	"CloseDoor",
			reinterpret_cast<PyCFunction>(CloseDoor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"OpenDoor",
			reinterpret_cast<PyCFunction>(OpenDoor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"GetDoorState",
			reinterpret_cast<PyCFunction>(GetDoorState),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"ToggleDoor",
			reinterpret_cast<PyCFunction>(ToggleDoor),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
#if (_DARKGAME == 2)
		{	"SetBlocking",
			reinterpret_cast<PyCFunction>(SetBlocking),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"GetSoundBlocking",
			reinterpret_cast<PyCFunction>(GetSoundBlocking),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
#endif
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDoorSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDoorSrv>::Name),
		sizeof(ScriptServiceObject<IDoorSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDoorSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDoorSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IDoorSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kDoorStateClosed", kDoorStateClosed);
		PyModule_AddIntConstant(module, "kDoorStateOpen", kDoorStateOpen);
		PyModule_AddIntConstant(module, "kDoorStateClosing", kDoorStateClosing);
		PyModule_AddIntConstant(module, "kDoorStateOpening", kDoorStateOpening);
		PyModule_AddIntConstant(module, "kDoorStateHalted", kDoorStateHalted);
	}

	ScriptServiceObject<IDoorSrv> * 
	ScriptService<IDoorSrv>::NewObject(PyObject *)
	{
		IDoorSrv *iface = static_cast<IDoorSrv*>(g_pScriptManager->GetService(IID_IDoorScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDoorSrv> *obj = PyObject_New(ScriptServiceObject<IDoorSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDoorSrv>::CloseDoor(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:CloseDoor", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->CloseDoor(obj));
	}

	PyObject * 
	ScriptService<IDoorSrv>::OpenDoor(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:OpenDoor", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OpenDoor(obj));
	}

	PyObject * 
	ScriptService<IDoorSrv>::GetDoorState(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:GetDoorState", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->GetDoorState(obj));
	}

	PyObject * 
	ScriptService<IDoorSrv>::ToggleDoor(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:ToggleDoor", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->ToggleDoor(obj));
	}

#if (_DARKGAME == 2)
	PyObject * 
	ScriptService<IDoorSrv>::SetBlocking(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		int on;
		if (!PyArg_ParseTuple(args, "O&i:SetBlocking", cvtObject, &obj, &on))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->SetBlocking(obj,on));
	}

	PyObject * 
	ScriptService<IDoorSrv>::GetSoundBlocking(ScriptServiceObject<IDoorSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:GetSoundBlocking", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->GetSoundBlocking(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
#endif


} // namespace PyNumbra


