/******************************************************************************
 *    LightScriptService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<ILightScrSrv>::Name = "LightScriptService";

	const char * 
	ScriptService<ILightScrSrv>::DocStrings[] = {
		"LightScriptService()\n\t"
			"An oxymoronic service, considering the name of the game engine.\n\t"
			"But players insist on being able to see what they're doing.",
		"Set(obj,mode,min,max)\n\t"
			"Sets the current mode of the light, and the minimum and\n\t"
			"maximum brightness values.",
		"SetMode(obj,mode)\n\t"
			"Changes the current mode of the light.",
		"Activate(obj)\n\t"
			"Clears the \"inactive\" flag. Does not change the light mode.",
		"Deactivate(obj)\n\t"
			"Sets the \"inactive\" flag, but does not change the mode.",
		"Subscribe(obj)",
		"Unsubscribe(obj)",
		"GetMode(obj) -> Integer\n\t"
			"Returns the current mode of the light.",
	};

	const PyMethodDef 
	ScriptService<ILightScrSrv>::Methods[] = {
		{	"Set",
			reinterpret_cast<PyCFunction>(Set),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"SetMode",
			reinterpret_cast<PyCFunction>(SetMode),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"Activate",
			reinterpret_cast<PyCFunction>(Activate),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"Deactivate",
			reinterpret_cast<PyCFunction>(Deactivate),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"Subscribe",
			reinterpret_cast<PyCFunction>(Subscribe),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"Unsubscribe",
			reinterpret_cast<PyCFunction>(Unsubscribe),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"GetMode",
			reinterpret_cast<PyCFunction>(GetMode),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<ILightScrSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<ILightScrSrv>::Name),
		sizeof(ScriptServiceObject<ILightScrSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<ILightScrSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<ILightScrSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<ILightScrSrv> * 
	ScriptService<ILightScrSrv>::NewObject(PyObject *)
	{
		ILightScrSrv *iface = static_cast<ILightScrSrv*>(g_pScriptManager->GetService(IID_ILightScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<ILightScrSrv> *obj = PyObject_New(ScriptServiceObject<ILightScrSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::Set(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		float min, max;
		int mode;
		object obj;
		if (!PyArg_ParseTuple(args, "O&iff:Set", cvtObject, &obj, &mode, &min, &max))
		{
			return NULL;
		}
		self->m_pIFace->Set(obj,mode,min,max);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::SetMode(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		int mode;
		object obj;
		if (!PyArg_ParseTuple(args, "O&i:SetMode", cvtObject, &obj, &mode))
		{
			return NULL;
		}
		self->m_pIFace->SetMode(obj,mode);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::Activate(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Activate", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->Activate(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::Deactivate(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Deactivate", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->Deactivate(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::Subscribe(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Subscribe", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->Subscribe(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::Unsubscribe(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Unsubscribe", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->Unsubscribe(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<ILightScrSrv>::GetMode(ScriptServiceObject<ILightScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:GetMode", cvtObject, &obj))
		{
			return NULL;
		}
		int ret = self->m_pIFace->GetMode(obj);
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


