/******************************************************************************
 *    LinkService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>
#include <lg/links.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<ILinkSrv>::Name = "LinkService";

	const char * 
	ScriptService<ILinkSrv>::DocStrings[] = {
		"LinkService()\n\t"
			"Provides an interface for working with links between objects.",
		"Create(kind,src,dest) -> PyNumbra.Link\n\t"
			"Adds a link of flavor kind from src to dest. None of the\n\t"
			"arguments may be 0. kind is an integer.",
		"Destroy(id) -> Integer\n\t"
			"Removes the specified link.",
		"AnyExist(kind,src,dest) -> Boolean\n\t"
			"Tests if there is at least one link of kind from src to dest.",
		"GetAll(kind,src,dest) -> PyNumbra.LinkIterator\n\t"
			"Returns an iterator over all links of flavor kind from src to\n\t"
			"dest. A value of 0 will match all links for that attribute.",
		"GetOne(kind,src,dest) -> PyNumbra.Link\n\t"
			"Retrieves the first link of kind from src to dest. Other links\n\t"
			"with matching attributes will be ignored.",
		"BroadcastOnAllLinks(src,msg,kind,data=None) -> Integer\n\t"
			"If data is specified, and the link flavor supports a data field,\n\t"
			"then the message will only be sent along links that match the\n\t"
			"specified data. It is unknown how link data with multiple fields\n\t"
			"is handled.",
		"CreateMany(kind,from,to) -> Integer\n\t"
			"from and to are strings.",
		"DestroyMany(kind,from,to) -> Integer\n\t"
			"from and to are strings.",
		"GetAllInherited(kind,src,dest) -> PyNumbra.LinkIterator",
		"GetAllInheritedSingle(kind,src,dest) -> PyNumbra.LinkIterator",
	};

	const PyMethodDef 
	ScriptService<ILinkSrv>::Methods[] = {
		{	"Create",
			reinterpret_cast<PyCFunction>(Create),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"Destroy",
			reinterpret_cast<PyCFunction>(Destroy),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"AnyExist",
			reinterpret_cast<PyCFunction>(AnyExist),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"GetAll",
			reinterpret_cast<PyCFunction>(GetAll),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"GetOne",
			reinterpret_cast<PyCFunction>(GetOne),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"BroadcastOnAllLinks",
			reinterpret_cast<PyCFunction>(BroadcastOnAllLinks),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"CreateMany",
			reinterpret_cast<PyCFunction>(CreateMany),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"DestroyMany",
			reinterpret_cast<PyCFunction>(DestroyMany),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"GetAllInherited",
			reinterpret_cast<PyCFunction>(GetAllInherited),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"GetAllInheritedSingle",
			reinterpret_cast<PyCFunction>(GetAllInheritedSingle),
			METH_VARARGS,
			const_cast<char*>(DocStrings[10])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<ILinkSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<ILinkSrv>::Name),
		sizeof(ScriptServiceObject<ILinkSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<ILinkSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<ILinkSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<ILinkSrv> * 
	ScriptService<ILinkSrv>::NewObject(PyObject *)
	{
		ILinkSrv *iface = static_cast<ILinkSrv*>(g_pScriptManager->GetService(IID_ILinkScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<ILinkSrv> *obj = PyObject_New(ScriptServiceObject<ILinkSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<ILinkSrv>::Create(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:Create", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		link ret;
		self->m_pIFace->Create(ret,kind,src,dest);
		if (ret == 0)
		{
			Exception::Raise(Exception::DarkEngineError, "could not create link");
			return NULL;
		}
		return LinkObject::New(ret);
	}

	PyObject * 
	ScriptService<ILinkSrv>::Destroy(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		link id;
		if (!PyArg_ParseTuple(args, "O&:Destroy", cvtLink, &id))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Destroy(id);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ILinkSrv>::AnyExist(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:AnyExist", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->AnyExist(ret,kind,src,dest);
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ILinkSrv>::GetAll(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:GetAll", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		linkset ls;
		self->m_pIFace->GetAll(ls,kind,src,dest);
		if (ls.query == NULL)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		// The query member of ls is set to NULL so it won't 
		// prematurely release the ILinkQuery.
		PyObject *ret = LinkIterObject::New(ls.query);
		ls.query = NULL;
		return ret;
	}

	PyObject * 
	ScriptService<ILinkSrv>::GetOne(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:GetOne", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		link ret;
		self->m_pIFace->GetOne(ret,kind,src,dest);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return LinkObject::New(ret);
	}

/* I'll just overload this
	PyObject * 
	ScriptService<ILinkSrv>::BroadcastOnAllLinks(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		const char *msg;
		object src;
		long kind;
		if (!PyArg_ParseTuple(args, "O&sl:BroadcastOnAllLinks", cvtObject, &src, &msg, &kind))
		{
			return NULL;
		}
		long ret = self->m_pIFace->BroadcastOnAllLinks(src,msg,kind);
		return PyInt_FromLong(ret);
	}
*/

	PyObject * 
	ScriptService<ILinkSrv>::BroadcastOnAllLinks(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		const char *msg;
		object src;
		long kind;
		cMultiParm data;
		PyObject *obData = NULL;
		if (!PyArg_ParseTuple(args, "O&sl|O:BroadcastOnAllLinks", cvtObject, &src, &msg, &kind, &obData))
		{
			return NULL;
		}
		long ret;
		if (obData && obData != Py_None)
		{
			if (!MultiParmFromObject(data, obData))
			{
				PyErr_SetString(PyExc_TypeError, "argument must be an Integer, Float, String, or Vector");
				return NULL;
			}
			ret = self->m_pIFace->BroadcastOnAllLinks(src,msg,kind,data);
			data.Free();
		}
		else
			ret = self->m_pIFace->BroadcastOnAllLinks(src,msg,kind);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ILinkSrv>::CreateMany(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		const char *from, *to;
		long kind;
		if (!PyArg_ParseTuple(args, "lss:CreateMany", &kind, &from, &to))
		{
			return NULL;
		}
		long ret = self->m_pIFace->CreateMany(kind,from,to);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ILinkSrv>::DestroyMany(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		const char *from, *to;
		long kind;
		if (!PyArg_ParseTuple(args, "lss:DestroyMany", &kind, &from, &to))
		{
			return NULL;
		}
		long ret = self->m_pIFace->DestroyMany(kind,from,to);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ILinkSrv>::GetAllInherited(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:GetAllInherited", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		linkset ls;
		self->m_pIFace->GetAllInherited(ls,kind,src,dest);
		if (ls.query == NULL)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		PyObject *ret = LinkIterObject::New(ls.query);
		ls.query = NULL;
		return ret;
	}

	PyObject * 
	ScriptService<ILinkSrv>::GetAllInheritedSingle(ScriptServiceObject<ILinkSrv> *self, PyObject *args)
	{
		object src, dest;
		long kind;
		if (!PyArg_ParseTuple(args, "lO&O&:GetAllInheritedSingle", &kind, cvtObject, &src, cvtObject, &dest))
		{
			return NULL;
		}
		linkset ls;
		self->m_pIFace->GetAllInheritedSingle(ls,kind,src,dest);
		if (ls.query == NULL)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		PyObject *ret = LinkIterObject::New(ls.query);
		ls.query = NULL;
		return ret;
	}


} // namespace PyNumbra


