/******************************************************************************
 *    LinkToolsService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<ILinkToolsSrv>::Name = "LinkToolsService";

	const char * 
	ScriptService<ILinkToolsSrv>::DocStrings[] = {
		"LinkToolsService()\n\t"
			"A few useful functions related to links. It's anyone's guess\n\t"
			"why this isn't part of LinkService.",
		"LinkKindNamed(name) -> Integer\n\t"
			"I haven't bother to make a special Python type just for link\n\t"
			"flavors. What you really want to do is use PyNumbra.LinkKindNamed.",
		"LinkKindName(kind) -> String\n\t"
			"Don't use this function. PyNumbra.LinkKindName is much better.",
		"LinkGet(id) -> Dictionary\n\t"
			"Retrieves the source, dest, and flavor of the link. The items\n\t"
			"are all Integers.",
		"LinkGetData(id,field=None) -> Integer, Float, String, Vector, or Boolean\n\t"
			"field should be None for simple structs.",
		"LinkSetData(id,field,data) -> Integer\n\t"
			"field should be None for simple structs.",
	};

	const PyMethodDef 
	ScriptService<ILinkToolsSrv>::Methods[] = {
		{	"LinkKindNamed",
			reinterpret_cast<PyCFunction>(LinkKindNamed),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"LinkKindName",
			reinterpret_cast<PyCFunction>(LinkKindName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"LinkGet",
			reinterpret_cast<PyCFunction>(LinkGet),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"LinkGetData",
			reinterpret_cast<PyCFunction>(LinkGetData),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"LinkSetData",
			reinterpret_cast<PyCFunction>(LinkSetData),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<ILinkToolsSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<ILinkToolsSrv>::Name),
		sizeof(ScriptServiceObject<ILinkToolsSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<ILinkToolsSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<ILinkToolsSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<ILinkToolsSrv> * 
	ScriptService<ILinkToolsSrv>::NewObject(PyObject *)
	{
		ILinkToolsSrv *iface = static_cast<ILinkToolsSrv*>(g_pScriptManager->GetService(IID_ILinkToolsScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<ILinkToolsSrv> *obj = PyObject_New(ScriptServiceObject<ILinkToolsSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<ILinkToolsSrv>::LinkKindNamed(ScriptServiceObject<ILinkToolsSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:LinkKindNamed", &name))
		{
			return NULL;
		}
		long ret = self->m_pIFace->LinkKindNamed(name);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ILinkToolsSrv>::LinkKindName(ScriptServiceObject<ILinkToolsSrv> *self, PyObject *args)
	{
		long kind;
		if (!PyArg_ParseTuple(args, "l:LinkKindName", &kind))
		{
			return NULL;
		}
		cScrStr ret;
		self->m_pIFace->LinkKindName(ret,kind);
		return PyString_FromString(static_cast<const char*>(ret));
	}

	PyObject * 
	ScriptService<ILinkToolsSrv>::LinkGet(ScriptServiceObject<ILinkToolsSrv> *self, PyObject *args)
	{
		link id;
		if (!PyArg_ParseTuple(args, "O&:LinkGet", cvtLink, &id))
		{
			return NULL;
		}
		sLink l;
		self->m_pIFace->LinkGet(id,l);
		return Py_BuildValue("{s:i,s:i,s:h}", "source", l.source, "dest", l.dest, "flavor", l.flavor);
	}

	PyObject * 
	ScriptService<ILinkToolsSrv>::LinkGetData(ScriptServiceObject<ILinkToolsSrv> *self, PyObject *args)
	{
		const char *field = NULL;
		link id;
		if (!PyArg_ParseTuple(args, "O&|z:LinkGetData", cvtLink, &id, &field))
		{
			return NULL;
		}
		cMultiParm data;
		self->m_pIFace->LinkGetData(data,id,field);
		if (data.type == kMT_Undef)
		{
			// either the arguments are invalid, or something went wrong internally
			// ehh... let the caller sort it out
			Py_INCREF(Py_None);
			return Py_None;
		}
		PyObject *ret = MultiParmToObject(data);
		data.Free(); // Wait... am I supposed to do this? I think so, but... 
		return ret;
	}

	PyObject * 
	ScriptService<ILinkToolsSrv>::LinkSetData(ScriptServiceObject<ILinkToolsSrv> *self, PyObject *args)
	{
		const char *field = NULL;
		link id;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&zO&:LinkSetData", cvtLink, &id, &field, cvtMultiParm, &data))
		{
			return NULL;
		}
		long ret = self->m_pIFace->LinkSetData(id,field,data);
		data.Free();
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


