/******************************************************************************
 *    NetworkingService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<INetworkingSrv>::Name = "NetworkingService";

	const char * 
	ScriptService<INetworkingSrv>::DocStrings[] = {
		"NetworkingService()\n\t"
			"Only available for System Shock 2.",
		"Broadcast(dest,msg,loop,data) -> Integer\n\t"
			"Sends a script message over the network. dest is the object the\n\t"
			"message will be sent to. msg is a string. If loop is 1, then the\n\t"
			"message will also be sent to the local object. data will be\n\t"
			"copied to the first data parameter of the message.",
		"SendToProxy(src,dest,msg,data) -> Integer\n\t"
			"Sends a message to another node on the network. src is the\n\t"
			"object the message will come from, as well as the host that\n\t"
			"the message will be sent to. dest is the object on that host\n\t"
			"which will receive the message. msg and data are as usual.",
		"TakeOver(obj) -> Integer\n\t"
			"Makes this proxy be the host for an object.\n\t",
		"GiveTo(obj,host) -> Integer\n\t"
			"Makes another proxy become the host for an object.",
		"IsPlayer(obj) -> Boolean\n\t"
			"Tests whether an object is one of the players.",
		"IsMultiplayer() -> Boolean\n\t"
			"Returns True if the game is in multiplayer mode.",
		"SetProxyOneShotTimer(obj,msg,time,data) -> Integer",
		"FirstPlayer() -> PyNumbra.Object\n\t"
			"Begins an iteration of all the player objects.",
		"NextPlayer() -> PyNumbra.Object\n\t"
			"Returns another player object, or 0 when there are no\n\t"
			"more players.",
		"Suspend() -> Integer\n\t"
			"Disables network communication temporarily.",
		"Resume() -> Integer\n\t"
			"Allows a network to begin communicating again.",
		"HostedHere(obj) -> Boolean\n\t"
			"Tests whether this host owns an object.",
		"IsProxy(obj) -> Boolean\n\t"
			"Tests whether an object is hosted by someone else.",
		"LocalOnly(obj) -> Boolean\n\t"
			"Tests whether the object only exists on this host.",
		"IsNetworking() -> Boolean\n\t"
			"Tests whether networking is active.",
	};

	const PyMethodDef 
	ScriptService<INetworkingSrv>::Methods[] = {
		{	"Broadcast",
			reinterpret_cast<PyCFunction>(Broadcast),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"SendToProxy",
			reinterpret_cast<PyCFunction>(SendToProxy),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"TakeOver",
			reinterpret_cast<PyCFunction>(TakeOver),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"GiveTo",
			reinterpret_cast<PyCFunction>(GiveTo),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"IsPlayer",
			reinterpret_cast<PyCFunction>(IsPlayer),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"IsMultiplayer",
			reinterpret_cast<PyCFunction>(IsMultiplayer),
			METH_NOARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"SetProxyOneShotTimer",
			reinterpret_cast<PyCFunction>(SetProxyOneShotTimer),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"FirstPlayer",
			reinterpret_cast<PyCFunction>(FirstPlayer),
			METH_NOARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"NextPlayer",
			reinterpret_cast<PyCFunction>(NextPlayer),
			METH_NOARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"Suspend",
			reinterpret_cast<PyCFunction>(Suspend),
			METH_NOARGS,
			const_cast<char*>(DocStrings[10])
		},
		{	"Resume",
			reinterpret_cast<PyCFunction>(Resume),
			METH_NOARGS,
			const_cast<char*>(DocStrings[11])
		},
		{	"HostedHere",
			reinterpret_cast<PyCFunction>(HostedHere),
			METH_VARARGS,
			const_cast<char*>(DocStrings[12])
		},
		{	"IsProxy",
			reinterpret_cast<PyCFunction>(IsProxy),
			METH_VARARGS,
			const_cast<char*>(DocStrings[13])
		},
		{	"LocalOnly",
			reinterpret_cast<PyCFunction>(LocalOnly),
			METH_VARARGS,
			const_cast<char*>(DocStrings[14])
		},
		{	"IsNetworking",
			reinterpret_cast<PyCFunction>(IsNetworking),
			METH_NOARGS,
			const_cast<char*>(DocStrings[15])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<INetworkingSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<INetworkingSrv>::Name),
		sizeof(ScriptServiceObject<INetworkingSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<INetworkingSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<INetworkingSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<INetworkingSrv> * 
	ScriptService<INetworkingSrv>::NewObject(PyObject *)
	{
		INetworkingSrv *iface = static_cast<INetworkingSrv*>(g_pScriptManager->GetService(IID_INetworkingScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<INetworkingSrv> *obj = PyObject_New(ScriptServiceObject<INetworkingSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::Broadcast(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		const char *msg;
		int loop;
		object dest;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&siO&:Broadcast", cvtObject, &dest, &msg, &loop, cvtMultiParm, &data))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Broadcast(dest,msg,loop,data);
		data.Free();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::SendToProxy(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		const char *msg;
		object src, dest;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&O&sO&:SendToProxy", cvtObject, &src, cvtObject, &dest, &msg, cvtMultiParm, &data))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SendToProxy(src,dest,msg,data);
		data.Free();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::TakeOver(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:TakeOver", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->TakeOver(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::GiveTo(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj, host;
		if (!PyArg_ParseTuple(args, "O&O&:GiveTo", cvtObject, &obj, cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->GiveTo(obj,host);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::IsPlayer(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsPlayer", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsPlayer(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::IsMultiplayer(ScriptServiceObject<INetworkingSrv> *self)
	{
		PyObject *ret = (self->m_pIFace->IsMultiplayer()) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::SetProxyOneShotTimer(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		const char *msg;
		object dest;
		cMultiParm data;
		float time;
		if (!PyArg_ParseTuple(args, "O&sfO&:SetProxyOneShotTimer", cvtObject, &dest, &msg, &time, cvtMultiParm, &data))
		{
			return NULL;
		}
		int ret = self->m_pIFace->SetProxyOneShotTimer(dest,msg,time,data);
		data.Free();
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::FirstPlayer(ScriptServiceObject<INetworkingSrv> *self)
	{
		object ret;
		self->m_pIFace->FirstPlayer(ret);
		if (ret)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::NextPlayer(ScriptServiceObject<INetworkingSrv> *self)
	{
		object ret;
		self->m_pIFace->NextPlayer(ret);
		if (ret)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<INetworkingSrv>::Suspend(ScriptServiceObject<INetworkingSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->Suspend());
	}

	PyObject * 
	ScriptService<INetworkingSrv>::Resume(ScriptServiceObject<INetworkingSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->Resume());
	}

	PyObject * 
	ScriptService<INetworkingSrv>::HostedHere(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:HostedHere", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->HostedHere(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::IsProxy(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsProxy", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsProxy(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::LocalOnly(ScriptServiceObject<INetworkingSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:LocalOnly", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->LocalOnly(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<INetworkingSrv>::IsNetworking(ScriptServiceObject<INetworkingSrv> *self)
	{
		PyObject *ret = (self->m_pIFace->IsNetworking()) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

} // namespace PyNumbra

#endif

