#!echo This script is for PyNumbra
'''PyDoor.py

    Implementation of the StdDoor script using PyNumbra.

    This file is part of PyNumbra
    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

from __future__ import division

# Every pyscript needs this
from PyNumbra import *
# Pull in a few constants.
from PyNumbra.ScriptServices import kDoorStateClosed,kDoorStateOpen,kDoorStateClosing,kDoorStateOpening,kDoorStateHalted
from PyNumbra.ScriptInterface import kDoorActionClose,kDoorActionOpen,kDoorActionClosing,kDoorActionOpening,kDoorActionHalt

# Action strings used for schemas.
DoorAction = {kDoorActionClose:'Closed', 
            kDoorActionOpen:'Open', 
            kDoorActionClosing:'Closing', 
            kDoorActionOpening:'Opening', 
            kDoorActionHalt:'Halted'
            }

# This is our script.
# PyNumbra expects the class name to be the same as
# the module name.
class PyDoor:
    
    # Send a SynchUp message to the door's mate.
    def PingDoubles(self):
        linksrv = ScriptServices.LinkService()
        # The LinkService is kind enough to filter based on the link data.
        # Note that we can use the global LinkKindNamed instead of going
        # through LinkToolsService.
        # Now, if you want to ignore the case of the link data,
        # you need to iterate the links manually.
        # See ScanDoubles for a typical iteration.
        linksrv.BroadcastOnAllLinks(self.m_iObjId,'SynchUp',LinkKindNamed('ScriptParams'),'Double')
        linksrv.BroadcastOnAllLinks(self.m_iObjId,'SynchUp',-LinkKindNamed('ScriptParams'),'Double')
        return
    
    # Returns the next state to transition to.
    # Taking full advantage of python objects here.
    def TargState(self,state):
        return {kDoorStateClosed:kDoorStateClosed, 
                kDoorStateOpen:kDoorStateOpen, 
                kDoorStateClosing:kDoorStateClose, 
                kDoorStateOpening:kDoorStateOpen, 
                kDoorStateHalted:kDoorStateHalted}[state]
    
    # Constructs the appropriate schema tags
    def StateChangeTags(self,status,oldstatus):
        tags = "Event StateChange, OpenState %s, OldOpenState %s" % (DoorAction[status], DoorAction[oldstatus])
        if ScriptInterface.IsScriptDataSet(self,self.m_iObjId,'PlayerFrob'):
            tags += ", CreatureType Player"
            # The flag is saved while opening or closing, so the
            # resultant open/close/halt schema will also be from
            # the Player.
            if status in (kDoorActionOpen,kDoorActionClose,kDoorActionHalt):
                ScriptInterface.ClearScriptData(self,self.m_iObjId,'PlayerFrob')
        return tags
    
    def StopCloseTimer(self,kill):
        if ScriptInterface.IsScriptDataSet(self,self.m_iObjId,'CloseTimer'):
            if kill:
                t = ScriptInterface.GetScriptData(self,self.m_iObjId,'CloseTimer')
                ScriptInterface.KillTimedMessage(t)
            ScriptInterface.ClearScriptData(self,self.m_iObjId,'CloseTimer')
        return
    
    # Make doors close automatically after being openned.
    def SetCloseTimer(self):
        propsrv = ScriptServices.PropertyService()
        if propsrv.Possessed(self.m_iObjId,'ScriptTiming'):
            self.StopCloseTimer(True)
            t = ScriptInterface.SetTimedMessage2(self.m_iObjId,'Close',propsrv.Get(self.m_iObjId,'ScriptTiming'))
            ScriptInterface.SetScriptData(self,self.m_iObjId,'CloseTimer',int(t))
        return
    
    # AI get confused when pathfinding through the
    # common point of double-doors. The AI will attempt
    # to frob both doors, which is essentially frobing
    # the same door twice.
    # So we create an unrendered object between the
    # doors to, hopefully, force AI to pathfind through
    # either one door or the other.
    def PathAvoidDoubles(self,door1,door2):
        # Only the door with the higher ID
        # will create the marker.
        if door2 > door1:
            # The pair of doors must share the same set of rooms.
            # This guards against linked doors that aren't actually
            # next to each other.
            rooms1 = self.GetRooms(door1)
            if not rooms1:
                return
            rooms2 = self.GetRooms(door2)
            if not rooms2:
                return
            if rooms1 != rooms2:
                return
            propsrv = ScriptServices.PropertyService()
            # Doors without physics? No thank you.
            if not propsrv.Possessed(door1,'PhysDims'):
                return
            if not propsrv.Possessed(door2,'PhysDims'):
                return
            # All doors should be OBBs.
            dim1 = propsrv.Get(door1,'PhysDims','Size')
            dim2 = propsrv.Get(door2,'PhysDims','Size')
            objsrv = ScriptServices.ObjectService()
            pos1 = objsrv.Position(door1)
            pos2 = objsrv.Position(door2)
            mid = map(lambda a,b: (a+b)/2, pos1, pos2)
            # Now test if the doors share a common midpoint.
            # note: sum([0.1]*10) == 0.1*10 is False!
            if abs(mid[0]-pos1[0]) - dim1[0] < 0.005 \
            or abs(mid[1]-pos1[1]) - dim1[1] < 0.005 \
            or abs(mid[2]-pos1[2]) - dim1[2] < 0.005:
                if abs(mid[0]-pos2[0]) - dim2[0] < 0.005 \
                or abs(mid[1]-pos2[1]) - dim2[1] < 0.005 \
                or abs(mid[2]-pos2[2]) - dim2[2] < 0.005:
                    sentinal = objsrv.Create(objsrv.Named('Marker'))
                    objsrv.Teleport(sentinal,mid,(0,0,0))
                    propsrv.Set(sentinal,'AI_ObjAvoid','Flags',2)
                    print "Created sentinal @" + str(mid)
        return
    
    # Iterate ScriptParams links to see if we need to create
    # path-avoid markers.
    def ScanDoubles(self):
        linksrv = ScriptServices.LinkService()
        linkcache = []
        # PathAvoidDoubles is called once for every linked object
        # we find. It's safe for both doors to do this, since
        # only the object with the greater ID creates the marker.
        ls = linksrv.GetAll(LinkKindNamed('ScriptParams'),self.m_iObjId,0)
        for l in ls:
            # You could squash the case here...
            if l.To() not in linkcache and l.GetData() == 'Double':
                linkcache.append(l.To())
                self.PathAvoidDoubles(self.m_iObjId,l.To())
        ls = linksrv.GetAll(LinkKindNamed('ScriptParams'),0,self.m_iObjId)
        for l in ls:
            if l.From() not in linkcache and l.GetData() == 'Double':
                linkcache.append(l.From())
                self.PathAvoidDoubles(self.m_iObjId,l.From())
        return
    
    # Returns a tuple of the rooms on either side of this door.
    # The tuple elements are sorted lowest - highest.
    def GetRooms(door):
        ret = None
        propsrv = ScriptServices.PropertyService()
        # It's actually not all that hard to get the field
        # name for a property: It's the label Dromed uses
        # for the dialog, spaces and all.
        if propsrv.Possessed(door,'RotDoor'):
            r1 = propsrv.Get(door,'RotDoor','Room ID #1')
            r2 = propsrv.Get(door,'RotDoor','Room ID #2')
            if r1 > r2:
                ret = (r2,r1)
            else:
                ret = (r1,r2)
        elif propsrv.Possessed(door,'TransDoor'):
            r1 = propsrv.Get(door,'TransDoor','Room ID #1')
            r2 = propsrv.Get(door,'TransDoor','Room ID #2')
            if r1 > r2:
                ret = (r2,r1)
            else:
                ret = (r1,r2)
        return ret
    GetRooms = staticmethod(GetRooms)
    
    # Now we get to the actual message handlers.
    # Handlers are called with a single argument, besides the
    # object instance: a dictionary of the message fields.
    # The keys of the dictionary can vary depending on what the
    # message is.
    # The handler should normally return True, or False if something
    # went wrong. (PyNumbra will catch any exceptions, so you can
    # let those go.) Or if a message is sent from a script that
    # expects a specific response, you can return a simple object.
    # (Simple means an integer, float, boolean, string, or 3-tuple)
    
    # StdDoor used a script variable named "Sim" directly.
    # This uses an instance variable instead. Look down at
    # the __getstate__ and __setstate__ methods to see how
    # the value is preserved.
    def OnSim(self,msg):
        if msg['fStarting']:
            self.ScanDoubles()
            self.m_bSim = True
        else:
            self.m_bSim = False
        return True
    
    # Scripts will get NowLocked/NowUnlocked even in editor-mode.
    def OnNowLocked(self,msg):
        if self.m_bSim:
            self.PingDoubles()
            doorsrv = ScriptServices.DoorService()
            doorsrv.CloseDoor(self.m_iObjId)
        return True
    
    def OnNowUnlocked(self,msg):
        if self.m_bSim:
            self.PingDoubles()
            doorsrv = ScriptServices.DoorService()
            doorsrv.OpenDoor(self.m_iObjId)
        return True
    
    def OnTimer(self,msg):
        if msg['name'] == 'Close':
            self.StopCloseTimer(False)
            doorsrv = ScriptServices.DoorService()
            doorsrv.CloseDoor(self.m_iObjId)
        return True
    
    def OnFrobWorldEnd(self,msg):
        doorsrv = ScriptServices.DoorService()
        # To save time, storing the service object is optional.
        if ScriptServices.LockService().IsLocked(self.m_iObjId) \
        and doorsrv.GetDoorState(self.m_iObjId) == kDoorStateClosed \
        and not ScriptServices.DarkGameService().ConfigIsDefined('LockCheat'):
            sndsrv = ScriptServices.SoundScriptService()
            sndsrv.PlayEnvSchema(self.m_iObjId, "Event Reject, Operation OpenDoor", self.m_iObjId, 0, 0)
        else:
            # Here's some real mojo. We can take call common methods on
            # game objects directly, like retrieving the SymName.
            if msg['Frobber'].Name() == 'Player':
                ScriptInterface.SetScriptData(self,self.m_iObjId,'PlayerFrob',0)
            if ScriptInterface.IsScriptDataSet(self,self.m_iObjId,'BeforeHalt'):
                beforehalt = ScriptInterface.GetScriptData(self,self.m_iObjId,'BeforeHalt')
                ScriptInterface.ClearScriptData(self,self.m_iObjId,'BeforeHalt')
                if beforehalt == kDoorActionOpening:
                    doorsrv.CloseDoor(self.m_iObjId)
                else:
                    doorsrv.OpenDoor(self.m_iObjId)
            else:
                doorsrv.ToggleDoor(self.m_iObjId)
        return True
    
    # PlayerToolFrob is sent by StdKey.
    def OnPlayerToolFrob(self,msg):
        ScriptInterface.SetScriptData(self,self.m_iObjId,'PlayerFrob',0)
        return True
    
    # After a door halts unexpectedly, we want a frob to do the reverse
    # of what it was doing before. Otherwise it would just get stuck
    # on whatever was making it halt.
    def OnDoorHalt(self,msg):
        # PingDoubles is not called. You would want to call it here to
        # simulate doors that are linked mechanically.
        ScriptInterface.SetScriptData(self,self.m_iObjId,'BeforeHalt',msg['PrevActionType'])
        sndsrv = ScriptServices.SoundScriptService()
        sndsrv.PlayEnvSchema(self.m_iObjId, self.StateChangeTags(msg['ActionType'],msg['PrevActionType']), self.m_iObjId, 0, 0)
        return True
    
    def OnDoorOpening(self,msg):
        self.PingDoubles()
        ScriptServices.DarkGameService().FoundObject(self.m_iObjId)
        sndsrv = ScriptServices.SoundScriptService()
        sndsrv.PlayEnvSchema(self.m_iObjId, self.StateChangeTags(msg['ActionType'],msg['PrevActionType']), self.m_iObjId, 0, 0)
        return True
    
    def OnDoorClosing(self,msg):
        self.PingDoubles()
        sndsrv = ScriptServices.SoundScriptService()
        sndsrv.PlayEnvSchema(self.m_iObjId, self.StateChangeTags(msg['ActionType'],msg['PrevActionType']), self.m_iObjId, 0, 0)
        self.StopCloseTimer(True)
        return True
    
    def OnDoorClose(self,msg):
        sndsrv = ScriptServices.SoundScriptService()
        sndsrv.HaltSchema(self.m_iObjId,"",0)
        sndsrv.PlayEnvSchema(self.m_iObjId, self.StateChangeTags(msg['ActionType'],msg['PrevActionType']), self.m_iObjId, 0, 0)
        return True
    
    def OnDoorOpen(self,msg):
        sndsrv = ScriptServices.SoundScriptService()
        sndsrv.HaltSchema(self.m_iObjId,"",0)
        sndsrv.PlayEnvSchema(self.m_iObjId, self.StateChangeTags(msg['ActionType'],msg['PrevActionType']), self.m_iObjId, 0, 0)
        self.SetCloseTimer()
        return True
    
    def OnSlain(self,msg):
        # Doors don't really "die". 
        ScriptServices.DamageService().Resurrect(self.m_iObjId,0)
        propsrv = ScriptServices.PropertyService()
        # Once slain, however, they will be much weaker than before.
        propsrv.Set(self.m_iObjId,'HitPoints',None,1)
        # Also, any locks are permanently disabled
        linksrv = ScriptServices.LinkService()
        ls = linksrv.GetAll(LinkKindNamed('Lock'),self.m_iObjId,0)
        for l in ls:
            linksrv.Destroy(l)
        if propsrv.Possessed(self.m_iObjId,'Locked'):
            # Just deleting the property doesn't actually unlock 
            # the door.
            propsrv.Set(self.m_iObjId,'Locked',None,0)
            propsrv.Remove(self.m_iObjId,'Locked')
        if propsrv.Possessed(self.m_iObjId,'KeyDst'):
            propsrv.Remove(self.m_iObjId,'KeyDst')
        ScriptServices.DoorService().OpenDoor(self.m_iObjId)
        return True
    
    def OnTurnOn(self,msg):
        doorsrv = ScriptServices.DoorService()
        doorsrv.OpenDoor(self.m_iObjId)
        return True
    
    def OnTurnOff(self,msg):
        doorsrv = ScriptServices.DoorService()
        doorsrv.CloseDoor(self.m_iObjId)
        return True
    
    # The double-door magic.
    # Mirror the behavior of the other door,
    # and copy it's lock state too.
    def OnSynchUp(self,msg):
        doorsrv = ScriptServices.DoorService()
        target = self.TargState(doorsrv.GetDoorState(msg['from']))
        state = self.TargState(doorsrv.GetDoorState(self.m_iObjId))
        if state != target:
            if target == kDoorStateClosed:
                doorsrv.CloseDoor(self.m_iObjId)
            else:
                doorsrv.OpenDoor(self.m_iObjId)
        propsrv = ScriptServices.PropertyService()
        # Notice that it doesn't look at Lock links.
        # You'll want to link your lock to both doors, in that case.
        if propsrv.Possessed(msg['from'],'Locked') and propsrv.Possessed(self.m_iObjId,'Locked'):
            if propsrv.Get(msg['from'],'Locked') != propsrv.Get(self.m_iObjId,'Locked'):
                propsrv.CopyFrom(self.m_iObjId, 'Locked', msg['from'])
        return True
    
    # PyNumbra reads this dictionary to know how to handle messages.
    # If a message isn't in the dictionary (or the dictionary doesn't
    # exist), then it will look for a method named 'OnMessage'.
    __Messages__ = {'Sim':OnSim, 
                    'NowLocked':OnNowLocked, 
                    'NowUnlocked':OnNowUnlocked, 
                    'Timer':OnTimer, 
                    'FrobWorldEnd':OnFrobWorldEnd, 
                    'PlayerToolFrob':OnPlayerToolFrob, 
                    'DoorHalt':OnDoorHalt, 
                    'DoorOpening':OnDoorOpening, 
                    'DoorClosing':OnDoorClosing, 
                    'DoorClose':OnDoorClose, 
                    'DoorOpen':OnDoorOpen, 
                    'Slain':OnSlain, 
                    'TurnOn':OnTurnOn, 
                    'Open':OnTurnOn, 
                    'TurnOff':OnTurnOff, 
                    'Close':OnTurnOff, 
                    'SynchUp':OnSynchUp, 
                   }
    
    # Standard python constructor.
    def __init__(self,objId):
        self.m_iObjId = objId
        self.m_bSim = False
    
    # All you need to do is return a tuple with the data
    # you want to save. PyNumbra serializes it to a string
    # and saves it as a ScriptVar named "_PickledScript".
    def __getstate__(self):
        return (self.m_iObjId, self.m_bSim)
    
    # And here we get back a tuple with the saved data.
    def __setstate__(self, persistence):
        (self.m_iObjId, self.m_bSim) = persistence
    
